/**
 * ============LICENSE_START========================================================================
 * ONAP : ccsdk feature sdnr wt odlux
 * =================================================================================================
 * Copyright (C) 2019 highstreet technologies GmbH Intellectual Property. All rights reserved.
 * =================================================================================================
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 * ============LICENSE_END==========================================================================
 */
import { IActionHandler } from '../../../../framework/src/flux/action';

import { MaintenenceEntry } from '../models/maintenenceEntryType';
import { LoadAllMainteneceEntriesAction, AllMainteneceEntriesLoadedAction, UpdateMaintenanceEntry } from '../actions/maintenenceActions';

export interface IMaintenenceEntriesState {
  entries: MaintenenceEntry[];
  busy: boolean;
}

const maintenanceEntriesStateInit: IMaintenenceEntriesState = {
  entries: [],
  busy: false
};

export const maintenenceEntriesActionHandler: IActionHandler<IMaintenenceEntriesState> = (state = maintenanceEntriesStateInit, action) => {
  if (action instanceof LoadAllMainteneceEntriesAction){
    state = {
      ...state,
      busy: true,
    }
  } else if (action instanceof AllMainteneceEntriesLoadedAction){
    state = {
      ...state,
      entries: action.maintenenceEntries || [],
      busy: false,
    }
  } else if (action instanceof UpdateMaintenanceEntry) {
    const index = state.entries.findIndex(e => action.maintenenceEntry.mountId === e.mountId);
    state = {
      ...state,
      entries: index > -1 ? [
        ...state.entries.slice(0, index),
        action.maintenenceEntry,
        ...state.entries.slice(index+1),
      ] : [...state.entries, action.maintenenceEntry]
    }
  }
  return state;
}